// ATRDialog.cpp : Implementation of CATRDialog

#include "stdafx.h"
#include "ATRDialog.h"
#include "ATRInterfaceApp.h"

// Defines
#define MAX_CTRL_TEXT_LEN     300       // Buffer length to keep window's title
#define CONTEXT_HELP_PREFIX   ".HELP"   // Prefix for the group where context help items are located

extern CATRInterfaceApp theApp;

// Global variables
#ifdef STRICT 
  WNDPROC           gpfParentWndProc;    ///< Pointer to parent window's window procedure
#else 
  FARPROC           gpfParentWndProc;    ///< Pointer to parent window's window procedure
#endif


/***************************************************************************************************/
//* CATRDialog::InitDialog
/***************************************************************************************************/
///
/// \brief Inits dialog box which is managed by the interface.
///
///   This must be called before dialog box is displayed. Dialog box is then translated and handle
///   to dialog box is stored. This function must be called before any other method in the
///   interface is called.
///
/// \param [in] lHWnd:  Handle to the dialog box which will be managed by this interface.
///
/// \param [in] bsGroup:  Group name from where to get translations for dialog box controls. If not
///                       specied the title of the dialog box is used instead.
///
/// \return S_OK if successful
///
/// \author tsalonen
///
/// \date 07.06.2006
/***************************************************************************************************/

STDMETHODIMP CATRDialog::InitDialog( LONG lHWnd, BSTR bsGroup )
{
  AFX_MANAGE_STATE(AfxGetStaticModuleState());

  EHSTART 

  bstr_t  bsGroupHelp(bsGroup);
 	TCHAR   acDialogTitle[MAX_CTRL_TEXT_LEN];

  mhParent = (HWND)lHWnd;

  if( 0 != GetWindowText( mhParent, acDialogTitle, MAX_CTRL_TEXT_LEN ) )
    moOrigDlgTitle = acDialogTitle;

  if( bsGroupHelp.length() == 0 )
  {
    // Use dialog box title as group
    moGroup = acDialogTitle;
  }
  else
  {
    moGroup = (LPCTSTR)bsGroupHelp;
  }

  // Construct moHelpGroup by adding .help at the end of moGroup
  moHelpGroup = moGroup + CString( CONTEXT_HELP_PREFIX );  

  // Translate dialog box controls
  TranslateDialogBox( );

  // Modify icons for OK, Cancel and Help buttons

  return S_OK;

  EHEND_STD 
}


/***************************************************************************************************/
//* CATRDialog::TranslateDialogBox
/***************************************************************************************************/
///
/// \brief Translates the dialog box title and all child controls recursively.
///
/// \author tsalonen
///
/// \date 07.06.2006
///
/// \sa EnumChildStatic( HWND hwnd, LPARAM lTransParam )
/***************************************************************************************************/

void CATRDialog::TranslateDialogBox()
{
  EHSTART 

	CString   oTranslation;
  
  if( mhParent == NULL )
  {
    ASSERT(0);
    return;
  }

  EnumChildWindows( mhParent, EnumChildStatic, (LPARAM)this );
	
	//translate the Dialog Caption
	oTranslation = GetTranslation( moGroup, DLGBOX_TITLE, moOrigDlgTitle );
	
	SetWindowTextW( mhParent, oTranslation );

  EHEND_NORET 
}


/***************************************************************************************************/
//* CATRDialog::EnumChildStatic
/***************************************************************************************************/
///
/// \brief Application-defined callback function used with the EnumChildWindows function
///
///   This is used to translate all child controls of specific window. This function recursively
///   traverses all child controls as well.
///
/// \param [in] hwnd:  Handle to a child window of the parent window specified in EnumChildWindows.
///
/// \param [in] lTransParam:  Specifies the application-defined value given in EnumChildWindows.
///
/// \return TRUE if successful; otherwise FALSE
///
/// \author tsalonen
///
/// \date 07.06.2006
///
/// \sa CATRDialog::EnumChild( HWND hwnd )
/***************************************************************************************************/

BOOL CALLBACK CATRDialog::EnumChildStatic( HWND hwnd, LPARAM lTransParam )
{
  AFX_MANAGE_STATE(AfxGetStaticModuleState());

  CATRDialog*  poDlg         = (CATRDialog*)lTransParam;

  if( poDlg )
  {
    // Check if I have children
    EnumChildWindows( hwnd, EnumChildStatic, (LPARAM)poDlg );
    return( poDlg->TranslateControl( hwnd ) );
  }

  return( FALSE );
}


/***************************************************************************************************/
//* CATRDialog::TranslateControl
/***************************************************************************************************/
///
/// \brief Translates specific control.
///
///   Translation is made based on control's ID. Translation is retrieved from specific group and
///   if translation is found it is set to control's text.
///
/// \param [in] hwnd:  Handle to the control whose text to translate.
///
/// \return TRUE if successful; otherwise FALSE
///
/// \author tsalonen
///
/// \date 07.06.2006
/***************************************************************************************************/

BOOL CALLBACK CATRDialog::TranslateControl( HWND hwnd )
{
  AFX_MANAGE_STATE(AfxGetStaticModuleState());

  EHSTART 

	CString   oTranslation;
  long      lKey            = 0;

  // Get window's id
  lKey = GetWindowLong( hwnd, GWL_ID );
	oTranslation = GetTranslation( moGroup, lKey );

  if( !oTranslation.IsEmpty() )
  {
	  SetWindowTextW( hwnd, oTranslation );
  }

  // Tell operating system to continue.
  return TRUE;

  EHEND_RET(FALSE);
}


/***************************************************************************************************/
//* CATRDialog::HelpWndProcStatic
/***************************************************************************************************/
///
/// \brief Application-defined function that processes messages sent to a window managed by this
///        interface.
///
///   Handles
///
/// \param [in] hwnd:  Handle to the window. 
///
/// \param [in] uMsg:  Specifies the message.
///
/// \param [in] wParam:  Specifies additional message information. The contents of this parameter
///                      depend on the value of the uMsg parameter. 
///
/// \param [in] lParam:  Specifies additional message information. The contents of this parameter
///                      depend on the value of the uMsg parameter. 
///
/// \return The return value is the result of the message processing and depends on the message
///         sent.
///
/// \author tsalonen
///
/// \date 07.06.2006
/***************************************************************************************************/

LRESULT CALLBACK CATRDialog::HelpWndProcStatic( HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
  AFX_MANAGE_STATE(AfxGetStaticModuleState());

  static CATRDialog* poDlg = NULL;

  if( poDlg == NULL )
  {
    poDlg = (CATRDialog*)GetWindowLong( hwnd, GWL_USERDATA );
  }

  if( poDlg != NULL  )
  {
    return( poDlg->HelpWndProc( hwnd, uMsg, wParam, lParam ) );
  }
  else
  {
    return( ::CallWindowProc( gpfParentWndProc, hwnd, uMsg, wParam, lParam ) );
  }  
}


/***************************************************************************************************/
//* CATRDialog::HelpWndProc
/***************************************************************************************************/
///
/// \brief Callback function called from static callback function.
///
///   Wraps window procedure to get window message associated to window managed by this interface.
///   Shows context sensitive help.
///
/// \param [in] hwnd:  Desired window style specified in ctDlgBoxStyle_e.
///
/// \param [in] uMsg:  Desired window style specified in ctDlgBoxStyle_e.
///
/// \param [in] wParam:  Desired window style specified in ctDlgBoxStyle_e.
///
/// \param [in] lParam:  Desired window style specified in ctDlgBoxStyle_e.
///
/// \return The return value is the result of the message processing and depends on the message
///         sent.
///
/// \author tsalonen
///
/// \date 09.06.2006
/***************************************************************************************************/

LRESULT CALLBACK CATRDialog::HelpWndProc( HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
  AFX_MANAGE_STATE(AfxGetStaticModuleState());

  LPHELPINFO      poHelpInfo;
	CString         oMessage;
  CString         oHelpGroup;
  long            lKey          = 0;


  switch( uMsg )
  {
  case WM_HELP:
    // Show context sensitive help message
    poHelpInfo = (LPHELPINFO)lParam;

    if( poHelpInfo->iContextType == HELPINFO_WINDOW )
    {
      lKey = poHelpInfo->iCtrlId;

      //retreive the message 
      oMessage = GetTranslation( moHelpGroup, lKey );

      // Load help text from String Table
      if( !oMessage.IsEmpty() )
      {
        HH_POPUP hPop; // HTML Help popup structure

        // Initialize structure to NULLs    
        memset(&hPop, 0, sizeof(hPop)); 

        // Set size of structure
        hPop.cbStruct         = sizeof(hPop);        

        // Yellow background color
        hPop.clrBackground    = RGB(255, 255, 208);    
        hPop.clrForeground    = -1; // Font color    
        hPop.rcMargins        = CRect(-1, -1, -1, -1);
        hPop.pt               = poHelpInfo->MousePos;    
        hPop.pszText          = oMessage; // Message from String Table
        hPop.pszFont          = NULL; // Font

        ::HtmlHelpW(hwnd, NULL, HH_DISPLAY_TEXT_POPUP, (DWORD)&hPop);
      } // End if found a help string for this request
      if( poHelpInfo->iContextType == HELP_CONTEXTPOPUP ) 
      {
        // AfxMessageBox("test", 0, 0);
      }
    }
    break;
  }

  return ::CallWindowProc(gpfParentWndProc, hwnd, uMsg, wParam, lParam);
}


/***************************************************************************************************/
//* CATRDialog::SetStyle
/***************************************************************************************************/
///
/// \brief Sets windows style.
///
///   Call this function to set window's style. It possible to specify e.g. how tooltips are shown,
///   is there context sensitive help available etc. InitDialog( LONG lHWnd, BSTR bsGroup ) must be
///   called before calling this function.
///
/// \param [in] lStyle:  Desired window style specified in ctDlgBoxStyle_e.
///
/// \return S_OK if successful; E_HANDLE if there isn't valid handle to the parent window.
///
/// \author tsalonen
///
/// \date 07.06.2006
/***************************************************************************************************/

STDMETHODIMP CATRDialog::SetStyle(LONG lStyle)
{
  AFX_MANAGE_STATE(AfxGetStaticModuleState());

  EHSTART

  long        lWindowStyle;

  // Check that init is already called
  if( mhParent == NULL )
  {
    return(E_HANDLE);
  }

  lWindowStyle = GetWindowLong( mhParent, GWL_EXSTYLE );

  // Consider context sensitive help
  if( lStyle & ctDlgboxStyleContextHelp  )
  {
    // Add ? mark button in the title bar. Note that it can't be
    // combined with maximized or minized button.
    lWindowStyle |= WS_EX_CONTEXTHELP;
    SetWindowLong( mhParent, GWL_EXSTYLE, lWindowStyle );
    SetWindowLong( mhParent, GWL_USERDATA, (LONG)this );

    // Change window procedure so that we get necessary messages
    // Get current window procedure
    gpfParentWndProc = 
#ifdef STRICT
    (WNDPROC)
#else
    (FARPROC)
#endif
    SetWindowLong( mhParent, GWL_WNDPROC, (LONG)HelpWndProcStatic );
  }
  else
  {
    // Remove possible ? mark providing context help
    lWindowStyle &= ~WS_EX_CONTEXTHELP;
    SetWindowLong( mhParent, GWL_EXSTYLE, lWindowStyle );
  }

  // Set bubble tooltips if desired

  return S_OK;

  EHEND_STD 
}